/*********************************************************************
题目：基于51单片机的微信消息推送，post模式
内容：通过MCU上位机对ESP8266wifi模块的控制和设置，实现stc89c51向微信推送消息
注意：晶振一定是11.0592MHz。
关于UID：在巴法云注册登陆，即可看到自己UID，推送微信消息，需要手机绑定微信，bemfa.com在控制台进行绑定即可。
QQ交流群：824273231
巴法云官网：bemfa.com
时间：2020/04/02
注意：新买的ESP8266-01默认是115200波特率，需要利用串口调试助手更改esp8266波特率为9600
			更改方法：USB转串口板连接到ESP8266，波特率115200，发送  AT+UART=9600,8,1,0,0   //末尾有回车换行
			 USB转串口板接线方法：tx--rx ,rx--tx,gnd-gnd,3.3v--3.3v,EN--3.3v //esp8266--USB转串口板
官方文档见官网：http://www.cloud.bemfa.com/docs/#/?id=_51-tcp%e5%88%9b%e5%ae%a2%e4%ba%91
********************************************************************/
#include <reg52.h>  
#include <stdio.h>
#include <string.h>//包含头文件
#define uint unsigned int
#define uchar unsigned char



#define Ssid  "newhtc"											//WIFI名称，，修改为自己路由器的WIFI名称，好像不支持中文
#define PassWord  "qq123456"              //WIFI密码，修改为自己路由器的
#define Uid  "4d9ec352e0376f2110a0c601a2857225" //巴法云UID密钥，控制台获取
#define Topic  "mytemp"          //巴法云控制台创建，名称自定义，app订阅主题要和这个推送主题一致



/*****************相关变量**************/
//uchar Receive,i;                             
//uint n;
//uchar Recive_table[15];        //用于接收wifi模块反馈到MCU上的数据
/*******************************************************************
名称：延时函数
作用：毫秒级延时，微妙级延时函数，为数据收发完成作等待.......
********************************************************************/
void ms_delay(uint t)
{
      uint i,j;
      for(i=t;i>0;i--)
       for(j=110;j>0;j--);
}

void us_delay(uchar t)
{
      while(t--);
}



/********************************************************************
名称：波特率发生器函数
作用：波特率发生器可以是T1定时器实现，也可以是MCU内部独立的波特率发生器，
各自不同的载入值计算式，具体根据寄存器相关设置来参考计算，以实现
异步串行通讯。（经测试，两种设置方式均可用，可任选一种。）
********************************************************************/
void Uart_Init()//使用定时器1作为波特率发生器（STC89C52、STC89C51、AT89C51或者STC12C560S2等均可）
{
      SCON=0x50;     //设置为串行口以方式1工作，8位异步通讯,允许接收中断。
      //一帧信息为10位，1位起始位，8位数据位（低位在先），1位停止位。
      PCON=0x80;     //SMOD波特率选择位为1，SMOD=1.
      TMOD=0x21;    //设置定时器1为波特率发生器，工作在模式2，8位自动装载
      TH1=0xFA ;//波特率9600 ，TH1=256-FOSC/16/12/波特率
      TL1=TH1;
      EA=1;                            //总中断打开
      ES=0;                             //关闭串口中断
      TR1=1;                     //启动定时器1
	
			TH0=0xD8;                  //定时10ms
			TL0=0xF0;
			ET0=1;
			TR0=1;
}




/********************************************************************
名称：串口发送函数
功能：MCU向其他与其连接的设备发送数据（此处是无线WIFI模块ESP8266)
********************************************************************/
void Send_Uart(uchar value)
{
      ES=0;         //关闭串口中断
      TI=0;         //清发送完毕中断请求标志位
      SBUF=value;     //发送
      while(TI==0);    //等待发送完毕
      TI=0;         //清发送完毕中断请求标志位
      ES=1;         //允许串口中断
}
/********************************************************************
名称：WIFI模块设置函数
作用: 启动模块，以便可以实现无线接入和控制
********************************************************************/
void ESP8266_Set(uchar *puf) // 数组指针*puf指向字符串数组               
{

      while(*puf!='\0')    //遇到空格跳出循环
      {
           Send_Uart(*puf);  //向WIFI模块发送控制指令。
           us_delay(5);
           puf++;      
      }
      us_delay(5);
      Send_Uart('\r');//回车
      us_delay(5);
      Send_Uart('\n');   //换行
}   

/********************************************************************
名称：WIFI模块设置函数
作用: 启动模块，没有回车换行
********************************************************************/
void ESP8266_Set_Only(uchar *puf) // 数组指针*puf指向字符串数组               
{

      while(*puf!='\0')    //遇到空格跳出循环
      {
           Send_Uart(*puf);  //向WIFI模块发送控制指令。
           us_delay(5);
           puf++;      
      }
}  



void POST(void)//POST数据函数
{
	int dataLen;
	uchar postData[150];
  char strLen[3];
	
	char data1[] = "32";             ////为了演示定义的模拟数据，可根据自己传感器自行采集数据
	char data2[] = "27.8";		//
	char data3[] = "45"; 
	char data4[] = "26"; 
	char data5[] = "99.12"; 
	char data6[] = "ON"; 
	
	//通过&符号整合需要post的数据。传感器数据用#分开，以便app进行数据分割
	//自己要知道每个数据代表的意思，一会app端会按排列分割提取数值
	sprintf(postData,"uid=%s&topic=%s&msg=#%s#%s#%s#%s#%s#%s#\r\n",Uid,Topic,data1,data2,data3,data4,data5,data6); 
	dataLen = strlen(postData);             //计算数据长度
  sprintf(strLen,"%d",dataLen); 					//int类型转char

  ESP8266_Set("AT+CIPMODE=1"); //开启透明传输模式
  ms_delay(1000);
  ESP8266_Set("AT+CIPSTART=\"TCP\",\"api.bemfa.com\",80");  // 连接服务器和端口
  ms_delay(1000);
  ESP8266_Set("AT+CIPSEND"); //进入透传模式，下面发的都会无条件传输
	ms_delay(1000);
	ESP8266_Set_Only("POST /api/device/v1/data/1/ HTTP/1.1\r\nHost: api.bemfa.com\r\nContent-Type: application/x-www-form-urlencoded\r\nConnection:close\r\nContent-Length:");//ţքdeviceۅ ţքsensorۅ
	ESP8266_Set(strLen);//数据长度（postData数据长度），一定要准确
	ESP8266_Set("");//回车（发完数据长度后，有回车，不懂的百度http post 格式）
	ESP8266_Set(postData);//发送数据	
	ms_delay(2000);
	ESP8266_Set("+++"); //
	memset(strLen,'\0',3);  //重置数组
	memset(postData,'\0',sizeof postData);//重置数组
}
/********************************************************************
名称：主函数
作用：程序的执行入口
********************************************************************/
void main()
{

      Uart_Init();//使用独立的波特率发生器
			ms_delay(1000);
      ESP8266_Set("AT+RST");     //重新启动wifi模块
			ms_delay(1000);
			ESP8266_Set("AT");     //重新启动wifi模块
			ms_delay(1000);
			ESP8266_Set("AT+CWMODE=3"); //设置路由器模式 1 station模式 2 AP路由器模式 3 station+AP混合模式
			ms_delay(1000);
      ESP8266_Set("AT+CWJAP=\""Ssid"\",\""PassWord"\"");//设置模块SSID:WIFI, PWD:密码 及安全类型加密模式（WPA2-PSK）
			ms_delay(8000);
      // ES=1;                                                           //允许串口中断
      while(1)
      {        
					POST();//可以设定满足什么条件是进行消息推送，这里只是一个demo
      }         
}

/*********************************************************************
名称：串行通讯中断
作用：发送或接收结束后进入该函数，对相应的标志位软件清0，实现模块对数
         据正常的收发。

void Uart_Interrupt() interrupt 4        
{

      if(RI==1)
      {
           RI=0;
           Receive=SBUF;        //MCU接收wifi模块反馈回来的数据
           Recive_table[ i]=Receive;      
           i++;         
           if((Recive_table[i-1]=='\n'))i=0;  //遇到换行 重新装值
}
      else TI=0;        
}
********************************************************************/